Package: spqn
Title: Spatial quantile normalization
Version: 1.24.0
Description: The spqn package implements spatial quantile normalization (SpQN). This method was
	     developed to remove a mean-correlation relationship in correlation matrices built from
	     gene expression data. It can serve as pre-processing step prior to a co-expression
	     analysis.
Authors@R: c(person("Yi", "Wang", role = c("cre", "aut"), email = "yiwangthu5@gmail.com"),
	     person(c("Kasper", "Daniel"), "Hansen", role = "aut"))
License: Artistic-2.0
Imports: graphics, stats, utils, matrixStats
Depends: R (>= 4.0), ggplot2, ggridges, SummarizedExperiment,
        BiocGenerics
Suggests: BiocStyle, knitr, rmarkdown, tools, spqnData (>= 0.99.3),
        RUnit
VignetteBuilder: knitr
URL: https://github.com/hansenlab/spqn
BugReports: https://github.com/hansenlab/spqn/issues
biocViews: NetworkInference, GraphAndNetwork, Normalization
git_url: https://git.bioconductor.org/packages/spqn
git_branch: RELEASE_3_23
git_last_commit: 0813c60
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.23
Date/Publication: 2026-04-28
NeedsCompilation: no
Packaged: 2026-04-29 00:59:40 UTC; biocbuild
Author: Yi Wang [cre, aut],
  Kasper Daniel Hansen [aut]
Maintainer: Yi Wang <yiwangthu5@gmail.com>

import("ggplot2")
import("ggridges")
import("SummarizedExperiment")
importFrom("graphics", "abline", "legend", "lines", "plot", "points")
importFrom("stats", "IQR", "cor", "density", "qqplot", "sd")
importFrom("utils", "tail")
importFrom("matrixStats", "rowMedians")

export("plot_signal_condition_exp",
       "get_IQR_condition_exp",
       "qqplot_condition_exp",
       "normalize_correlation",
       "plot_IQR_condition_exp")

library(spqn)
library(spqnData)
library(SummarizedExperiment)

data(gtex.4k)
cor_m <- cor(t(assay(gtex.4k)))
ave_logrpkm <- rowData(gtex.4k)$ave_logrpkm

cor_m_spqn <- normalize_correlation(cor_m, ave_exp=ave_logrpkm, ngrp=20, size_grp=300, ref_grp=18)

cor_m_spqn_sample = cor_m_spqn[1:500,1:500]
save(cor_m_spqn_sample, file=file.path("..", "unitTests", "cor_m_spqn_sample.rda"))

sessionInfo()

## ----load, message=FALSE------------------------------------------------------
library(spqn)
library(spqnData)
library(SummarizedExperiment)

## ----cor_m, message=FALSE-----------------------------------------------------
data(gtex.4k)
cor_m <- cor(t(assay(gtex.4k)))
ave_logrpkm <- rowData(gtex.4k)$ave_logrpkm

## ----examplePreproc, eval=FALSE-----------------------------------------------
# # only keep coding genes and lincRNA in the counts matrix
# gtex <- gtex[rowData(gtex)$gene_type %in%
#                           c("lincRNA", "protein_coding"), ]
# 
# # normalize the counts matrix by log2rpkm
# cSums <- colSums(assay(gtex))
# logrpkm <- sweep(log2(assay(gtex) + 0.5), 2, FUN = "-",
#                  STATS = log2(cSums / 10^6))
# logrpkm <- logrpkm - log2(rowData(gtex)$gene_length / 1000)
# 
# # only keep those genes with median log2rpkm above 0
# wh.expressed  <- which(rowMedians(logrpkm) > 0)
# 
# gtex.0pcs <- gtex[wh.expressed,]
# logrpkm.0pcs <- logrpkm[wh.expressed,]
# ave_logrpkm <- rowMeans(logrpkm.0pcs)
# logrpkm <- logrpkm - ave_logrpkm # mean centering
# logrpkm  <- logrpkm / matrixStats::rowSds(logrpkm) # variance scaling
# assays(gtex.0pcs) <- SimpleList(logrpkm = logrpkm.0pcs)
# rowData(gtex.0pcs)$ave_logrpkm <- ave_logrpkm
# 
# # remove PCs from the gene expression matrix after scaling each gene to have mean=0 and variance=1
# gtex.4pcs <- gtex.0pcs
# assay(gtex.4pcs) <- removePrincipalComponents(t(scale(t(logrpkm.0pcs))), n = 4)

## ----message=FALSE------------------------------------------------------------
plot_signal_condition_exp(cor_m, ave_logrpkm, signal=0)

## ----message = FALSE----------------------------------------------------------
plot_signal_condition_exp(cor_m, ave_logrpkm, signal=0.001)

## ----message=FALSE------------------------------------------------------------
IQR_list <- get_IQR_condition_exp(cor_m, rowData(gtex.4k)$ave_logrpkm)
plot_IQR_condition_exp(IQR_list)

## ----message = FALSE----------------------------------------------------------
IQR_unlist <- unlist(lapply(1:10, function(ii) IQR_list$IQR_cor_mat[ii, ii:10]))
plot(rep(IQR_list$grp_mean, times = 1:10),
     IQR_unlist,
     xlab="min(average(log2RPKM))", ylab="IQR", cex.lab=1.5, cex.axis=1.2, col="blue")

## ----message = FALSE----------------------------------------------------------
par(mfrow = c(3,3))
for(j in c(1:8,10)){
    qqplot_condition_exp(cor_m, ave_logrpkm, j, j)
}

## ----spqn, message = FALSE----------------------------------------------------
cor_m_spqn <- normalize_correlation(cor_m, ave_exp=ave_logrpkm, ngrp=20, size_grp=300, ref_grp=18)

## ----message = FALSE----------------------------------------------------------
plot_signal_condition_exp(cor_m_spqn, ave_logrpkm, signal=0)

## ----message = FALSE----------------------------------------------------------
plot_signal_condition_exp(cor_m_spqn, ave_logrpkm, signal=0.001)

## ----message=FALSE------------------------------------------------------------
IQR_spqn_list <- get_IQR_condition_exp(cor_m_spqn, rowData(gtex.4k)$ave_logrpkm)
plot_IQR_condition_exp(IQR_spqn_list)

## ----message = FALSE----------------------------------------------------------
par(mfrow = c(3,3))
for(j in c(1:8,10)){
    qqplot_condition_exp(cor_m_spqn, ave_logrpkm, j, j)
}

## ----message = FALSE----------------------------------------------------------
IQR_unlist <- unlist(lapply(1:10, function(ii) IQR_spqn_list$IQR_cor_mat[ii, ii:10]))
plot(rep(IQR_spqn_list$grp_mean, times = 1:10),
     IQR_unlist,
     xlab="min(average(log2RPKM))", ylab="IQR", cex.lab=1.5, cex.axis=1.2, col="blue")

## ----sessionInfo--------------------------------------------------------------
sessionInfo()

.get_grps <- function(cor_mat, ngrp=20, size_grp=400){
    ngene <- nrow(cor_mat)
    idx <- seq_len(ngene-size_grp+1)
    grp_label <- cut(idx, ngrp-1)
    grp_loc0 <- split(idx, grp_label)
    
    if(size_grp-length(grp_loc0[[1]])<5){
        idx <- seq_len(ngene)
        grp_label <- cut(idx, ngrp)
        grp_loc <- split(idx, grp_label)
    }else{
        grp_loc <- vector(mode = "list", length = ngrp)
        idx <- seq_len(ngene-size_grp+1)
        grp_label <- cut(idx, ngrp-1)
        grp_loc0 <- split(idx, grp_label)
        for(i in seq_len(ngrp-1)){
            id1 <- grp_loc0[[i]][1]
            id2 <- grp_loc0[[i]][1]+size_grp-1
            grp_loc[[i]] <- c(id1:id2)
        }
        id1 <- ngene-size_grp+1
        grp_loc[[ngrp]] <- c(id1:ngene)
    }
    grp_loc
}


##### Asssign inner bins
## In each running group, get a inner group
.get_grps_inner <- function(grp_loc){
    grp_loc_inner <- list()
    ngrp <- length(grp_loc)
    size_bin <- length(grp_loc[[1]])
    ngene <- max(grp_loc[[ngrp]])
    
    width_tmp <- grp_loc[[2]][1] - grp_loc[[1]][1]
    length_inner_1 <- round(size_bin/2 + width_tmp/2)
    grp_loc_inner[[1]] <- seq_len(length_inner_1)
    
    for(i in 2:(ngrp-1)){
        id1 <- grp_loc[[i+1]][1]
        id2 <- grp_loc[[i]][1]
        width_tmp <- id1 - id2
        tail <- tail(grp_loc_inner[[i-1]], 1)
        grp_loc_inner[[i]] <- (tail + 1):(tail + width_tmp)
    }
    tail <- tail(grp_loc_inner[[ngrp-1]],1)
    grp_loc_inner[[ngrp]] <- c((tail+1) : ngene)
    
    grp_loc_inner
}


##### Get rank for each running bin
.get_bin_rank <- function(cor_obs, grp_loc, grp_loc_inner, cor_ref){
    rank_bin <- rank_bin_pre <- array(dim=dim(cor_obs))
    ngrp <- length(grp_loc)
    size_bin <- length(grp_loc[[1]])
    
    l_cor_tmp_ref <- length(cor_ref[upper.tri(cor_ref)])
    
    for(i in seq_len(ngrp)){
        for(j in i:ngrp){
            cor_bin_tmp <- cor_obs[grp_loc[[i]],grp_loc[[j]]]
            rank_bin_tmp <- array(dim=dim(cor_bin_tmp))
            l_cor_tmp <- length(cor_bin_tmp)
            
            idx <- seq_len(l_cor_tmp)
            rank_bin_tmp[idx] <- rank(cor_bin_tmp[idx])
            
            ## Number of diagonals(of full correlation matrix) contained in the bin
            n_diag <- sum(grp_loc[[i]] %in% grp_loc[[j]])
            
            ## Scale the rank of each bin to same scale as rank(cor_ref),
            ## After scaling, rank_bin could contain non-integars
            rank_bin_pre[grp_loc[[i]], grp_loc[[j]]] <- rank_bin_tmp
            rank_replace <- rank_bin_pre[grp_loc_inner[[i]], grp_loc_inner[[j]]]
            # scale the rank to range [0,1]
            nreplace <- l_cor_tmp-n_diag-1
            rank_replace <- (rank_replace-1)/ nreplace
            # scale the rank to the same scale as the referent bin
            rank_replace <- 1 + (rank_replace * (l_cor_tmp_ref-1))
            # change the ranks that larger than the size of referent bin, to be the size of referent bin
            rank_replace[which(rank_replace>l_cor_tmp_ref)] <- l_cor_tmp_ref
            
            # rank_bin_pre[grp_loc_inner[[i]],grp_loc_inner[[j]]] <-  rank_replace
            # rank_bin_pre[grp_loc_inner[[i]],grp_loc_inner[[j]]][which(rank_replace>l_cor_tmp_ref)] <- l_cor_tmp_ref
       
            rank_bin[grp_loc_inner[[i]],grp_loc_inner[[j]]] <- rank_replace
            # rank_bin[grp_loc_inner[[i]],grp_loc_inner[[j]]] <- rank_bin_pre[grp_loc_inner[[i]],grp_loc_inner[[j]]]
        }
    }
    
    rank_bin
}

##### Transform rank to cor_est
.est_cor <- function(rank_bin, cor_ref){
    cor_adj <- array(dim=dim(rank_bin))
    
    cor_ref_upper_tri <- cor_ref[upper.tri(cor_ref)]
    cor_ref_sorted <- sort(cor_ref_upper_tri)
    
    up_tri <- upper.tri(cor_adj)
    
    ## Find to nearest integars to rank_bin, since rank_bin could contain non-integars
    rank_bin <- rank_bin[up_tri]
    rank_bin1 <- rank_bin %/% 1

    ## Assign weights to each rank according to the distance to rank_bin
    rank_bin_w2 <- (rank_bin - rank_bin1)

    rank_bin2 <- rank_bin1+1
    rank_bin_w1 <- 1-rank_bin_w2

    ## Find the correlations in the cor_ref corresponding to the two nearest ranks to rank_bin
    ## Estimate the correlation using weighted average based on the distance to rank_bin
    length_cor_ref <- length(cor_ref_sorted)
    rank_bin2[which(rank_bin2>length_cor_ref)] <- length_cor_ref
    cor_adj[up_tri] <- rank_bin_w1*cor_ref_sorted[rank_bin1]+ rank_bin_w2*cor_ref_sorted[rank_bin2]

    low_tri <- lower.tri(cor_adj)
    cor_adj[low_tri] <- t(cor_adj)[low_tri]
    diag(cor_adj) <- 1
    cor_adj
}
normalize_correlation <- function(cor_mat, ave_exp, ngrp, size_grp, ref_grp){
  ## stopifnot(is.integer(ngrp), is.integer(size_grp), is.integer(ref_grp))
  stopifnot(length(ngrp) == 1, length(size_grp) == 1, length(ref_grp) == 1)
  stopifnot(ref_grp <= ngrp)
  stopifnot(0 < ngrp, 0 < size_grp, 0 < ref_grp)
  stopifnot(is.matrix(cor_mat))
  stopifnot(nrow(cor_mat) == ncol(cor_mat), nrow(cor_mat) == length(ave_exp))
  
  idx <- order(ave_exp)
  ngene <- length(ave_exp)
  ref_vec <- seq_len(ngene)
  
  cor_mat <- cor_mat[idx, idx]
  ref_vec_rearranged <- ref_vec[idx]
  
  group_loc <- .get_grps(cor_mat, ngrp, size_grp)
  
  ## Asssign inner bins
  group_loc_adj <- .get_grps_inner(group_loc)
  
  ## Get rank for each running bin
  cor_ref <- cor_mat[group_loc[[ref_grp]], group_loc[[ref_grp]]]
  rank_bin <- .get_bin_rank(cor_mat, group_loc, group_loc_adj, cor_ref)
  
  ## Transform rank to cor_adj
  cor_est <- .est_cor(rank_bin, cor_ref)
  
  idx <- order(ref_vec_rearranged)
  cor_est <- cor_est[idx, idx]
  cor_est
}

test_normalize_correlation <- function(){
  load(file.path(path.package("spqn"), "unitTests", "cor_m_spqn_sample.rda"))
  data(gtex.4k)
  cor_m <- cor(t(assay(gtex.4k)))
  ave_logrpkm <- rowData(gtex.4k)$ave_logrpkm
  cor_m_spqn_test <- normalize_correlation(cor_m, ave_exp=ave_logrpkm, ngrp=20, size_grp=300, ref_grp=18)
  idx <- seq_len(500)
  cor_m_spqn_test_sample <- cor_m_spqn_test[idx, idx]
  checkEquals(cor_m_spqn_sample, cor_m_spqn_test_sample)
}

 

require("spqn") || stop("unable to load the spqn package")
require("spqnData") || stop("unable to load the spqnData package")
require("SummarizedExperiment") || stop("unable to load the SummarizedExperiment package")
BiocGenerics:::testPackage("spqn")

get_grp_loc <- function(cor_matrix, ngrp=10){
    ngene <- nrow(cor_matrix)
    idx <- seq_len(ngene)
    grp_label <- cut(idx, ngrp)
    grp_loc <- split(idx, grp_label)
    return(grp_loc)
}

get_IQR_condition_exp <- function(cor_mat, ave_exp){
    idx <- order(ave_exp)
    cor_mat <- cor_mat[idx, idx]
    ave_exp <- ave_exp[idx]
    grp_loc <- get_grp_loc(cor_mat)
    IQR_cor_mat <- array(dim=c(10,10))
    grp_mean <- array(dim=10)
    for(i in seq_len(10)) {
        cor_tmp <- cor_mat[grp_loc[[i]],grp_loc[[i]]]
        cor_tmp_up_tri <- cor_tmp[upper.tri(cor_tmp)]
        IQR_cor_mat[i,i] <- IQR(cor_tmp_up_tri)
        if(i < 10){ 
            for(j in (i+1):10) {
                cor_tmp <- cor_mat[grp_loc[[i]],grp_loc[[j]]]
                IQR_cor_mat[i,j] <- IQR(cor_tmp)
            }
        }
        ave_exp_grp <- ave_exp[grp_loc[[i]]]
        grp_mean[i] <-  mean(ave_exp_grp)
    }
    idx <- lower.tri(IQR_cor_mat)
    IQR_cor_mat[idx] <- t(IQR_cor_mat)[idx]
    return(list(IQR_cor_mat=IQR_cor_mat,
                grp_mean=grp_mean))
}


qqplot_condition_exp <- function(cor_mat, ave_exp, i, j){
    idx <- order(ave_exp)
    cor_mat <- cor_mat[idx, idx]
    group_loc <- get_grp_loc(cor_mat)
    cor_ref <- cor_mat[group_loc[[9]], group_loc[[9]]]
    cor_ref_vec <- cor_ref[upper.tri(cor_ref)]
    cor_tmp <- cor_mat[group_loc[[i]], group_loc[[j]]]
    if(i==j) {
        cor_tmp <- cor_tmp[upper.tri(cor_tmp)]
    }
    qqplot(cor_ref_vec, as.vector(cor_tmp),
           xlab="reference correlations - (9,9)",
           ylab=paste0("correlations in (",i,",",j,")"),
           cex.lab=1, cex.axis=1, cex=0.2)
    abline(0, 1, col="blue")
}

plot_signal_condition_exp <- function(cor_mat, ave_exp, signal) { 
  idx <- order(ave_exp)
  cor_mat <- cor_mat[idx, idx]
  if(isTRUE(all.equal(signal, 0))){
    ngrp <- 10
    ngene <- ncol(cor_mat)
    idx <- seq_len(ngene)
    grp_label <- cut(idx, ngrp)
    grp_loc <- split(idx, grp_label) 
    
    length_group <- lapply(grp_loc, length)
    length_group <- unlist(length_group)
    length_group <- length_group*(length_group-1)/2
    length_group_cumulate <- cumsum(length_group)
    
    cor_vec_all <- data.frame(matrix(ncol = 2, 
                                     nrow = length_group_cumulate[10]))
    colnames(cor_vec_all) <- c("correlation",  "group")
    
    for(i in seq_len(10)) {
      cor_tmp <- cor_mat[grp_loc[[i]], grp_loc[[i]]]
      cor_tmp <- cor_tmp[upper.tri(cor_tmp)]
      cor_tmp <- as.numeric(cor_tmp)
      if(i==1){
        idx1 <- 1
      }else{
        idx1 <- length_group_cumulate[i-1]+1
      }
      idx2 <- length_group_cumulate[i]
      idx <- c(idx1 : idx2)
      cor_vec_all$correlation[idx] <- cor_tmp
      length_tmp <- length_group[i]
      cor_vec_all$group[idx] <- rep(i,length_tmp)
    }
    cor_vec_all <- data.frame(cor_vec_all)
    names(cor_vec_all) <- c("correlation", "group")
    cor_vec_all$group <- as.factor(cor_vec_all$group)
    ggplot(cor_vec_all, aes_string(x = "correlation", y = "group")) +
      geom_density_ridges2(fill="blue") +
      theme_ridges( grid = TRUE) + theme(axis.title.x = element_blank()) + 
      geom_vline(xintercept = 0, linetype="dotted", color = "black", size=0.3)
  }else{
    ncor <- length(which(upper.tri(cor_mat)))
    nsignal <- ncor*signal
    nsignal_grp <- round(nsignal/100)
    
    # list_cor_sig <- list_cor_back <- grp_back <- grp_sig <- c()
    grp_locs <- get_grp_loc(cor_mat)
    list_cor_sig <- grp_sig <- numeric(10*nsignal_grp)
    nbackground_group <- lapply(grp_locs, length)
    nbackground_group <- unlist(nbackground_group)
    nbackground_group <- nbackground_group*(nbackground_group-1)/2
    list_cor_back_cumulate <- cumsum(nbackground_group)
    list_cor_back <- grp_back <- numeric(list_cor_back_cumulate[10])
    for(ngrp in seq_len(10)) {
      loc_back <- grp_locs[[ngrp]]
      cor_back <- cor_mat[loc_back,loc_back]
      cor_back <- cor_back[upper.tri(cor_back)]
      order_cor_ori_grp <- order(abs(cor_back), decreasing=TRUE)
      idx <- seq_len(nsignal_grp)
      id_signal <- order_cor_ori_grp[idx]
      cor_signal_ori <- cor_back[id_signal]
      
      idx1 <- (ngrp-1)*nsignal_grp+1
      idx2 <- ngrp*nsignal_grp
      idx <- c(idx1 : idx2)
      list_cor_sig[idx] <- cor_signal_ori
      grp_sig[idx] <- ngrp
      
      if(ngrp==1){
        idx <- seq_len(list_cor_back_cumulate[ngrp])
        list_cor_back[idx] <- cor_back
        grp_back[idx] <- ngrp
      }else{
        idx1 <- list_cor_back_cumulate[ngrp-1]+1
        idx2 <- list_cor_back_cumulate[ngrp]
        idx <- c(idx1:idx2)
        list_cor_back[idx] <- cor_back
        grp_back[idx] <- ngrp }                                                                                                       
    }
    df_sig <- data.frame(correlation=list_cor_sig, bin=grp_sig, group="signal")
    df_back <- data.frame(correlation=list_cor_back, 
                          bin=grp_back, group="background")
    df_merge <- rbind(df_sig,df_back)
    df_merge$bin_group  <-  paste(df_merge$bin, df_merge$group)
    ggplot(df_merge, aes_string(y = "bin")) +
      geom_density_ridges(
        aes_string(x = "correlation", fill = "bin_group"),
        alpha = .8, color = "white") +
      labs(x = "correlation", y = "bin")  +
      scale_y_discrete(limits=seq_len(10)) +
      scale_fill_cyclical(
        labels = c("background","signal"),
        values = c("#0000ff", "#ff0000"),
        name = "group", guide = "legend") +
      theme_ridges(grid = FALSE)
  }
}


plot_IQR_condition_exp <- function(IQR_list){
  IQR_cor_mat <- IQR_list$IQR_cor_mat
  grp_mean <- IQR_list$grp_mean
  IQR_cor_mat2 <- round(IQR_cor_mat, 3)
  IQR_cor_mat <- IQR_cor_mat/max(IQR_cor_mat)
  max_IQR <- max(IQR_cor_mat)
  margin <- min(IQR_cor_mat)/10
  sd_grps_offset_y <- t(matrix(rep(max_IQR,100), nrow=10))
  for(i in 2:10){
    sd_grps_offset_y[i,] <- max_IQR + sd_grps_offset_y[i-1,] + margin
  }
  sd_grps_offset_x=t(matrix(rep(max_IQR,100),nrow=10))
  for(i in 2:10){
    sd_grps_offset_x[,i]=max_IQR+sd_grps_offset_y[i-1,]+margin
  }
  sd <- as.numeric(IQR_cor_mat)
  x1 <- as.numeric((-IQR_cor_mat/2)+sd_grps_offset_x)
  x2 <- as.numeric((IQR_cor_mat/2)+sd_grps_offset_x)
  y1 <- as.numeric((-IQR_cor_mat/2)+sd_grps_offset_y)
  y2 <- as.numeric((IQR_cor_mat/2)+sd_grps_offset_y)
  IQR_cor_mat <- round(IQR_cor_mat,3)
  d <- data.frame(x1,x2,y1,y2,sd)
  
  ggplot() + 
    geom_rect(data=d, mapping=aes(xmin=x1, xmax=x2, ymin=y1, ymax=y2), 
              color="black", alpha=0) +
    xlab("average(log2RPKM)")+ylab("average(log2RPKM)") +
    scale_y_discrete(limits=c(seq_len(10))+c(0:9)*min(IQR_cor_mat)/10,
                     labels=round(grp_mean,1)) +
    scale_x_discrete(limits=c(seq_len(10))+c(0:9)*min(IQR_cor_mat)/10,
                     labels=round(grp_mean,1)) + theme_bw() +
    theme(axis.title.x = element_text(size=20),axis.title.y = element_text(size=20),
          axis.text=element_text(size=20))
}

test_get_grps <- function(){
  data(grp_loc_target)
  load(file.path(path.package("spqn"), "unitTests", "grp_loc_target.rda"))

  grp_loc_assigend = spqn:::get_grp_loc(cor_mat=array(dim=c(100,100)),ngrp=10)
  grp_loc_assigend =  matrix(unlist(grp_loc_assigend), ncol = 10, byrow = TRUE)
    
  checkEquals(grp_loc_target, grp_loc_assigend)
}

\name{plot_IQR_condition_exp}
\alias{plot_IQR_condition_exp}
\alias{get_IQR_condition_exp}
\title{
  Get and plot the IQRs of submatrices of the correlation matrix.
}
\description{
  The \code{get_IQR_condition_exp} function computes the IQRs of a set
  of 10 by 10 same-size bins that partition the correlation matrix,
  ordered according to expression level.
  
  The \code{plot_IQR_condition_exp} function plots the IQR for each bin
  among a set of 10 by 10 same-size bins that partition the correlation
  matrix, with IQR denoted by the width of boxes in the plot.
}
\usage{
get_IQR_condition_exp(cor_mat, ave_exp)
plot_IQR_condition_exp(IQR_list)
}
\arguments{
\item{cor_mat}{correlation matrix, generated by gene expression matrix,
  with genes sorted by average expression levels.}
\item{ave_exp}{vector, average expression level of each gene for the
  normalized gene expression matrix.}
\item{IQR_list}{List, output of \code{get_IQR_condition_exp}.}
}
\note{
  The mnemonic for \code{condition_exp} is \sQuote{conditional on
    expression}.
}
\value{
  A plot with boxes that shows the IQR of each bin
}
\examples{
if(require(spqnData)) {
    data(gtex.4k)
    cor_mat <- cor(t(assay(gtex.4k)))
    ave_logrpkm <- rowData(gtex.4k)$ave_logrpkm
    IQR_list <- get_IQR_condition_exp(cor_mat, ave_exp = ave_logrpkm)
    plot_IQR_condition_exp(IQR_list)
    }
}

\name{normalize_correlation}
\alias{normalize_correlation}
\title{Spatial quantile normalization (SpQN)}
\description{
  This method was developed to remove a mean-correlation relationship in
  correlation matrices built from gene expression data. It can serve as
  pre-processing step prior to a co-expression analysis.
}
\usage{
normalize_correlation(cor_mat, ave_exp, ngrp, size_grp, ref_grp)
}
\arguments{
  \item{cor_mat}{A (square and symmetrix) correlation matrix.}
  \item{ave_exp}{A vector of expression levels, same length as the
    number of rows of the correlation matrix in \code{cor_mat}. For
    other types of data, \code{ave_exp} can be the vector corresponding to the
    row/column of the correlation matrix, whose dependency with the
    distribution of correlations need to be removed.}
  \item{ngrp}{Number of bins in each row/column to be used to
    partition the correlation matrix, integer.}
  \item{size_grp}{Size of the outer bins to be used to appriximate the
    distribution of the inner bins, in order to smooth the
    normalization. Note that the product of size_grp and ngrp must be
    equal or larger than than the row/column number of cor_mat, and
    there is no smoothness in the normalization when they are equal.}
  \item{ref_grp}{Location of the reference bin on the diagonal, whose
    distribution will be used as target distribution in the
    normalization, an integer.}
}
\value{
  A normalized correlation matrix.
}
\examples{
if(require(spqnData)){
  data(gtex.4k)
  cor_ori <- cor(t(assay(gtex.4k)))
  ave_logrpkm <- rowData(gtex.4k)$ave_logrpkm
  normalize_correlation(cor_ori, ave_exp = ave_logrpkm,
                        ngrp=10, size_grp=15, ref_grp=9)}
}

\name{qqplot_condition_exp}
\alias{qqplot_condition_exp}
\title{
  Q-Q plot for examing the distributions across submatrices of a
  correlation matrix.
}
\description{
  We partition the correlation matrix into 10x10 bins of equal size,
  with genes ordered according to expression level. As reference bin, we
  choose the (9,9) bin (ie. the almost-highest expressed genes). We then
  make a QQ-plot of the (i,j)'th submatrix vs. the (9,9) submatrix. See
  the SpQN paper for detail on these choices.
}
\usage{
qqplot_condition_exp(cor_mat,ave_exp, i,j)
}
\arguments{
  \item{cor_mat}{Matrix, correlation matrix, generated by gene expression matrix.}
  \item{ave_exp}{Vector, average expression level of each gene for the normalized expression matrix.}
  \item{i}{Integer, row number of the submatrix (see details).}
  \item{j}{Integer, column number of the submatrix (see details).}
}
\note{
  The mnemonic for \code{condition_exp} is \sQuote{conditional on
    expression}.
}
\value{
  Invoked for the side effect of producing a plot.
}
\examples{
if(require(spqnData)) {
  data(gtex.4k)
  cor_mat <- cor(t(assay(gtex.4k)))
  ave_logrpkm <- rowData(gtex.4k)$ave_logrpkm
  qqplot_condition_exp(cor_mat, ave_exp=ave_logrpkm, 1, 1)
}
}

\name{plot_signal_condition_exp}
\alias{plot_signal_condition_exp}
\title{
  Plot the signal and background distribution of a correlation matrix.
}
\description{
  This function allows users to visualize the distributions of (assumed)
  signal and background, conditional on expression levels.  The
  predicted signals are defined by the 0.1\% highest correlations in
  each bin.
}
\usage{
plot_signal_condition_exp(cor_mat, ave_exp, signal)
}
\arguments{
  \item{cor_mat}{Matrix, correlation matrix, generated by gene
    expression matrix}
  \item{ave_exp}{Vector, average expression level of each gene for the
    normalized expression matrix}
  \item{signal}{a value between 0 and 1 giving the fraction of
    correlations which should be considered signal. We often use a value
    of \code{0.001}.}
}
\note{
  The mnemonic for \code{condition_exp} is \sQuote{conditional on
    expression}.
}
\value{
  Invoked for the side effect of producing a plot.
}
\examples{
if(require(spqnData)) {
  data(gtex.4k)
  cor_mat <- cor(t(assay(gtex.4k)))
  ave_logrpkm <- rowData(gtex.4k)$ave_logrpkm
  plot_signal_condition_exp(cor_mat, ave_exp=ave_logrpkm, signal=0.05)}
}

\name{spqn-package}
\alias{spqn-package}
\alias{spqn}
\docType{package}
\title{
\packageTitle{spqn}
}
\description{
\packageDescription{spqn}
}
\details{
  See references for details on spatial quantile normalization.

  The main function is \code{normalize_correlation}. We include a number
  of plotting functions for examining the mean-correlation relationship,
  see the vignette for examples.
  
}
\references{
  Y Wang, SC Hicks, KD Hansen (2020).
  \emph{Co-expression analysis is biased by a mean-correlation relationship.}
  bioRxiv 2020.02.13.944777. \doi{10.1101/2020.02.13.944777}
}
\keyword{ package }
